/******************************************************************
Copyright 2000 by Object Craft P/L, Melbourne, Australia.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Object Craft
is not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

OBJECT CRAFT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
EVENT SHALL OBJECT CRAFT BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

******************************************************************/

#include "Python.h"
#include "structmember.h"

#define FORCE_PROTOTYPES
#ifdef _WIN32
# define DBNTWIN32
# include <windows.h>
# include "sqlfront.h"
# include "sqldb.h"
# define SQLMAXNAME MAXNAME
# define DBBOOL BOOL
#else
# include "sybfront.h"
# include "sybdb.h"
# define SMALLDATETIBIND SMALLDATETIMEBIND
# define FLT4BIND REALBIND
# define DBFLT4 DBREAL
# define DBDATETIM4 DBDATETIME4
# ifdef MAXNAME
#  define SQLMAXNAME MAXNAME
# else
#  define SQLMAXNAME DBMAXNAME
# endif
# define SQLVOID      SYBVOID
# define SQLTEXT      SYBTEXT
# define SQLVARBINARY SYBVARBINARY
# define SQLINTN      SYBINTN
# define SQLVARCHAR   SYBVARCHAR
# define SQLBINARY    SYBBINARY
# define SQLIMAGE     SYBIMAGE
# define SQLCHAR      SYBCHAR
# define SQLINT1      SYBINT1
# define SQLBIT       SYBBIT
# define SQLINT2      SYBINT2
# define SQLINT4      SYBINT4
# define SQLMONEY     SYBMONEY
# define SQLDATETIME  SYBDATETIME
# define SQLFLT8      SYBFLT8
# define SQLFLTN      SYBFLTN
# define SQLMONEYN    SYBMONEYN
# define SQLDATETIMN  SYBDATETIMN
# define SQLFLT4      SYBREAL
# define SQLMONEY4    SYBMONEY4
# define SQLDATETIM4  SYBDATETIME4
# define SQLDECIMAL   SYBDECIMAL
# define SQLNUMERIC   SYBNUMERIC
#endif

enum { TYPE_RETCODE, TYPE_ANSWER, TYPE_ROWRES, TYPE_VALUE,
       TYPE_SERVTYPE, TYPE_RPCOPT, TYPE_RPCPARAM, TYPE_BIND,
};

void debug_msg(char *fmt, ...);
char *value_str(int type, int value);

typedef struct {
    PyObject_HEAD

    LOGINREC *login;
    int debug;
} LOGINRECObj;

extern PyTypeObject LOGINRECType;
PyObject *LOGINREC__init__(int debug);

typedef struct DBPROCObj {
    PyObject_HEAD

    DBPROCESS *dbproc;
    int debug;
    struct DBPROCObj *next;
} DBPROCObj;

extern PyTypeObject DBPROCType;
PyObject *dbproc_find_object(DBPROCESS *db_dbproc);
PyObject *DBPROC__init__(LOGINREC *login, char *server, int debug);

typedef struct {
    PyObject_HEAD
    int strip;
    int vartype;
    DBINT varlen;
    DBINT size;
    BYTE *buff;
} DataBufObj;

extern PyTypeObject DataBufType;
#define DataBuf_Check(obj) (obj->ob_type == &DataBufType)
int is_fixedlen(int vartype);
PyObject *DataBuf_convert(DBPROCObj *dbproc, PyObject *from, DataBufObj *to);
PyObject *DataBuf__init__(int vartype, int varlen);

typedef struct {
    PyObject_HEAD
    DBNUMERIC num;
} NumericObj;

#define NUMERIC_LEN PRNUMERIC

extern PyTypeObject NumericType;
#define Numeric_Check(obj) (obj->ob_type == &NumericType)
PyObject *numeric_alloc(DBNUMERIC *num);
int numeric_as_string(PyObject *obj, char *text);
extern char numeric_new__doc__[];
PyObject *NumericType_new(PyObject *module, PyObject *args);
void copy_reg_numeric(PyObject *dict);
extern char pickle_numeric__doc__[];
PyObject *pickle_numeric(PyObject *module, PyObject *args);

typedef struct {
    PyObject_HEAD
    DBDATETIME datetime;
    DBDATEREC daterec;
    int cracked;
} DateTimeObj;

#define DATETIME_LEN 32

extern PyTypeObject DateTimeType;
#define DateTime_Check(obj) (obj->ob_type == &DateTimeType)
PyObject *datetime_alloc(DBDATETIME *num);
int datetime_as_string(PyObject *obj, char *text);
extern char datetime_new__doc__[];
PyObject *DateTimeType_new(PyObject *module, PyObject *args);
void copy_reg_datetime(PyObject *dict);
extern char pickle_datetime__doc__[];
PyObject *pickle_datetime(PyObject *module, PyObject *args);

typedef struct {
    PyObject_HEAD
    DBMONEY money;
} MoneyObj;

#define MONEY_LEN 32

extern PyTypeObject MoneyType;
#define Money_Check(obj) (obj->ob_type == &MoneyType)
PyObject *money_alloc(DBMONEY *num);
int money_as_string(PyObject *obj, char *text);
extern char money_new__doc__[];
PyObject *MoneyType_new(PyObject *module, PyObject *args);
void copy_reg_money(PyObject *dict);
extern char pickle_money__doc__[];
PyObject *pickle_money(PyObject *module, PyObject *args);
